# Reference

- [Reference](#reference)
  - [Block Reference](#block-reference)
    - [Server Block](#server-block)
    - [Files Block](#files-block)
    - [SPA Block](#spa-block)
    - [API Block](#api-block)
    - [Endpoint Block](#endpoint-block)
    - [Proxy Block](#proxy-block)
    - [Request Block](#request-block)
    - [Response Block](#response-block)
    - [Backend Block](#backend-block)
      - [Duration](#duration)
    - [OpenAPI Block](#openapi-block)
    - [CORS Block](#cors-block)
    - [OAuth2 Block](#oauth2-block)
    - [Definitions Block](#definitions-block)
    - [Basic Auth Block](#basic-auth-block)
    - [JWT Block](#jwt-block)
    - [JWT Signing Profile Block](#jwt-signing-profile-block)
    - [SAML Block](#saml-block)
    - [Settings Block](#settings-block)
  - [Access Control](#access-control)
  - [Health-Check](#health-check)
  - [Variables](#variables)
    - [env](#env)
    - [request](#request)
    - [backend_requests](#backend_requests)
    - [backend_responses](#backend_responses)
  - [Functions](#functions)
  - [Modifier](#modifier)
    - [Request Header](#request-header)
    - [Response Header](#response-header)
    - [Set Response Status](#set-response-status)
  - [Parameters](#parameters)
    - [Query Parameter](#query-parameter)
    - [Form Parameter](#form-parameter)
    - [Path Parameter](#path-parameter)

## Block Reference

### Server Block

The `server` block is one of the root configuration blocks of Couper's configuration file.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
|-|&#9888; required| [CORS Block](#cors-block), [Files Block](#files-block), [SPA Block](#spa-block) , [API Block(s)](#api-block), [Endpoint Block(s)](#endpoint-block) |

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------  | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `base_path`     |string|-|Configures the path prefix for all requests.| &#9888; Inherited by nested blocks.|`base_path = "/api"` |
|`hosts`|list|port `:8080`|-|&#9888; required, if there is more than one  [Server Block](#server-block). &#9888; Only one `hosts` attribute per  [Server Block](#server-block) is allowed.| `hosts = ["example.com", "localhost:9090"]`|
|`error_file`|string|-|Location of the error file template.|-|`error_file = "./my_error_page.html"`|
|`access_control`|list|-|Sets predefined [Access Control](#access-control) for [Server Block](#server-block) context.|&#9888; Inherited by nested blocks.|`access_control = ["foo"]`|

### Files Block

The `files` block configures the file serving.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
|[Server Block](#server-block)|no label| [CORS Block](#cors-block)|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------  | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
|`base_path`|string|-|Configures the path prefix for all requests.|-|`base_path = "/files"`|
|`document_root`|string|-|-|&#9888; required|`document_root = "./htdocs"`|
|`error_file`|string|-|Location of the document root.|-| -|
|`access_control` |list|-|Sets predefined [Access Control](#access-control) for [Files Block](#files-block) context.|-|`access_control = ["foo"]`|

### SPA Block

The [SPA Block](#spa-block) configures the Web serving for SPA assets.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Server Block](#server-block)|no label|[CORS Block](#cors-block) |

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------  | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
|`base_path`|string|-|Configures the path prefix for all requests.|-|`base_path = "/assets"`|
|`bootstrap_file`|string|-|Location of the bootstrap file.|&#9888; required| `bootstrap_file = "./htdocs/index.html"`|
|`paths`|list|-|list of SPA paths that need the bootstrap file.|&#9888; required|`paths = ["/app/**"]`|
|`access_control` |list|-|Sets predefined [Access Control](#access-control) for [SPA Block](spa-block) context.|-|`access_control = ["foo"]`|

### API Block

The [API Block](#api-block) bundles `endpoints` under a certain `base_path`.

&#9888; If an error occurred for api endpoints the response gets processed
as json error with an error body payload. This can be customized via `error_file`.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
|[Server Block](#server-block)|Optional| [Endpoint Block(s)](#endpoint-block),  [CORS Block](#cors-block)|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------  | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
|`base_path`|string|-|Configures the path prefix for all requests.|&#9888; Must be unique if multiple [API Blocks](#api-block) are defined.| `base_path = "/v1"`|
| `error_file` |string|-|Location of the error file template.|-|`error_file = "./my_error_body.json"`|
| `access_control` |list|-|Sets predefined [Access Control](#access-control) for [API Block](#api-block) context.|&#9888; Inherited by nested blocks.| `access_control = ["foo"]`|

### Endpoint Block

[Endpoint Block](#endpoint-block)s define the entry points of Couper. The required _label_
defines the path suffix for the incoming client request. The `path` attribute
changes the path for the outgoing request (compare
[path mapping example](./README.md#routing-path-mapping)). Each [Endpoint Block](#endpoint-block) must
produce an explicit or implicit client response.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Server Block](#server-block), [API Block](#api-block) |&#9888; required, defines the path suffix for incoming client requests | [Proxy Block(s)](#proxy-block),  [Request Block(s)](#request-block), [Response Block](#response-block) |

<!-- TODO: decide how to place "modifier" in the reference table - same for other block which allow modifiers -->

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
|`request_body_limit`  |string|`64MiB`|Configures the maximum buffer size while accessing `request.form_body` or `request.json_body` content.|&#9888; Valid units are: `KiB, MiB, GiB`|`request_body_limit = "200KiB"`|
| `path`|string|-|Changeable part of the upstream URL. Changes the path suffix of the outgoing request.|-|-|
|`access_control`   |list|-|Sets predefined [Access Control](#access-control) for [Endpoint Block](#endpoint-block) context.|-| `access_control = ["foo"]`|
|[Modifier](#modifier) |-|-|-|-|-|

### Proxy Block

The [Proxy Block](#proxy-block) creates and executes a proxy request to a backend service.

&#9888; Multiple [Proxy](#proxy-block) and [Request Block](#request-block)s are executed in parallel.
<!-- TODO: shorten label text in table below and find better explanation for backend, backend reference or url - same for request block-->

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
|[Endpoint Block](#endpoint-block)|&#9888; A [Proxy Block](#proxy-block) or [Request Block](#request-block) w/o a label has an implicit label `"default"`. Only **one** [Proxy Block](#proxy-block) or [Request Block](#request-block) w/ label `"default"` per [Endpoint Block](#endpoint-block) is allowed.|[Backend Block](#backend-block) (&#9888; required, if no [Backend Block](#backend-block) reference is defined or no `url` attribute is set.)|-|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `backend` |string|-|[Backend Block](#backend-block) reference, defined in [Definitions Block](#definitions-block)|&#9888; required, if no [Backend Block](#backend-block) or `url` attribute is defined.|`backend = "foo"`|
| `url` |string|-|If defined, the host part of the URL must be the same as the `origin` attribute of the [Backend Block](#backend-block) (if defined).|-|-|-|
|[Modifier](#modifier)|-|-|-|-|-|

### Request Block

The [Request Block](#request-block) creates and executes a request to a backend service.

&#9888; Multiple [Proxy](#proxy-block) and [Request Block](#request-block)s are executed in parallel.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Endpoint Block](#endpoint-block)|&#9888; A [Proxy Block](#proxy-block) or [Request Block](#request-block) w/o a label has an implicit label `"default"`. Only **one** [Proxy Block](#proxy-block) or [Request Block](#request-block) w/ label `"default"` per [Endpoint Block](#endpoint-block) is allowed.|[Backend Block](#backend-block) (&#9888; required, if no `backend` block reference is defined or no `url` attribute is set.|
<!-- TODO: add available http methods -->
| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `backend` |string|-|`backend` block reference, defined in [Definitions Block](#definitions-block)|&#9888; required, if no [Backend Block](#backend-block) is defined.|`backend = "foo"`|
| `url` |string|-|-|If defined, the host part of the URL must be the same as the `origin` attribute of the used [Backend Block](#backend-block) or [Backend Block Reference](#backend-block) (if defined).|-|-|
|`body`|string|-|-| Creates implicit default `Content-Type: text/plain` header field.|-|
|`json_body`|null, bool, number, string, map, list|-|-|Creates implicit default `Content-Type: text/plain` header field.|-|
| `form_body` |map|-|-|Creates implicit default `Content-Type: application/x-www-form-urlencoded` header field.|-|
|`method`    |string|`GET`|-|-|-|
|`headers`  |-|-|-|Same as `set_request_headers` in [Request Header](#request-header).|-|
|`query_params`|-|-|-|Same as `set_query_params` in [Query Parameter](#query-parameter).|-|                                                                                                                                                         - |

### Response Block

The [Response Block](#response-block) creates and sends a client response.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
|[Endpoint Block](#endpoint-block).|no label|-|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `body`         | string|-|-|Creates implicit default `Content-Type: text/plain` header field.|-|                                              |
| `json_body`    | null, bool, number, string, map, list|-|-|Creates implicit default `Content-Type: application/json` header field.|-|-|
| `status`       | integer|`200`|HTTP status code.|-|-|                                                                                 |
| `headers`      |string|-|Same as `set_response_headers` in [Request Header](#response-header).                  |-|-|

### Backend Block

A [Backend Block](#backend-block) defines the connection to a local/remote backend service.

&#9888; Backends can be defined in the [Definitions Block](#definitions-block) and referenced by _label_.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Definitions Block](#definitions-block), [Proxy Block](#proxy-block), [Request Block](#request-block)| &#9888; required, when defined in [Definitions Block](#definitions-block)| [OpenAPI Block](#openapi-block), [OAuth2 Block](#oauth2-block)|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `basic_auth`                    | string|-|Basic auth for the upstream request. | format is `username:password`|-|
| `hostname`                      | string |-|Value of the HTTP host header field for the origin request. |Since `hostname` replaces the request host the value will also be used for a server identity check during a TLS handshake with the origin.|-|
| `origin`                        |string|-|URL to connect to for backend requests.|&#9888; required.  &#9888; Must start with the scheme `http://...`.|-|                                                                                          |
| `path`                          | string|-|Changeable part of upstream URL.|-|-|                                                                                                                                                                       |
| `path_prefix`  | string|-|Prefixes all backend request paths with the given prefix|&#9888; Must start with the scheme `http://...`. |-|-|                                                                                    |
| `connect_timeout`                | [duration](#duration) | `10s`      | The total timeout for dialing and connect to the origin.   |-                                   |-|
| `disable_certificate_validation` | bool               | `false`       | Disables the peer certificate validation.                                              |      - |-|
| `disable_connection_reuse`       | bool               | `false`        | Disables reusage of connections to the origin.                                          |    -  |-|
| `http2`                          | bool               | `false`         | Enables the HTTP2 support.                                                               | -    |-|
| `max_connections`                | integer                | `0` (unlimited) | The maximum number of concurrent connections in any state (_active_ or _idle_) to the origin. |-|-|
| `proxy`                          | string             | -| A proxy URL for the related origin request.      |-   | `http://SERVER-IP_OR_NAME:PORT`|
| `timeout`                        | [duration](#duration) | `300s`          | The total deadline duration a backend request has for write and read/pipe.               |-     |-|
| `ttfb_timeout`                   | [duration](#duration) | `60s`           | The duration from writing the full request to the origin and receiving the answer.        |-    |-|
| [Modifier](#modifier)           |- |-|All [Modifier](#modifier)|-|-|                                                                                                       |

#### Duration

| Duration units | Description  |
| :-------------- | :----------- |
| `ns`            | nanoseconds  |
| `us` (or `µs`)  | microseconds |
| `ms`            | milliseconds |
| `s`             | seconds      |
| `m`             | minutes      |
| `h`             | hours        |

### OpenAPI Block

The [OpenAPI Block](#openapi-block)configures the backends proxy behaviour to validate outgoing
and incoming requests to and from the origin. Preventing the origin from invalid
requests, and the Couper client from invalid answers. An example can be found
[here](https://github.com/avenga/couper-examples/blob/master/backend-validation/README.md).
To do so Couper uses the [OpenAPI 3 standard](https://www.openapis.org/) to load
the definitions from a given document defined with the `file` attribute.

&#9888; While ignoring request violations an invalid method or path would
lead to a non-matching _route_ which is still required for response validations.
In this case the response validation will fail if not ignored too.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Backend Block](#backend-block)|-|-|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `file`                       |string|-|OpenAPI yaml definition file.|&#9888; required|-|                                     |
| `ignore_request_violations`  |bool|`false`|Log request validation results, skip error handling. |-|-|
| `ignore_response_violations` |bool|`false`|Log response validation results, skip error handling.|-|-|

### CORS Block

The [CORS Block](#cors-block) configures the CORS (Cross-Origin Resource Sharing) behavior in Couper.

<!--TODO: check if this information is correct -->
&#9888; Overrides the CORS behavior of the parent block-

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
|[Server Block](#server-block), [Files Block](#files-block), [SPA Block](#spa-block), [API Block](#api-block).  |no label|-|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `allowed_origins`   | string|-|A list of allowed origin(s).|Can be either of: a string with a single specific origin, `"*"` (all origins are allowed) or an array of specific origins | `allowed_origins = ["https://www.example.com", "https://www.another.host.org"]`|
| `allow_credentials` |bool|`false`| Set to `true` if the response can be shared with credentialed requests (containing `Cookie` or `Authorization` HTTP header fields).|-|-|                                                                                                                                                         |
| `disable`           | bool|`false`|Set to `true` to disable the inheritance of CORS from the [Server Block](#server-block) in [Files Block](#files-block), [SPA Block](#spa-block) and [API Block](#api-block) contexts.|-|-|                                                                                                       |
| `max_age`           |[duration](#duration)|-|Indicates the time the information provided by the `Access-Control-Allow-Methods` and `Access-Control-Allow-Headers` response HTTP header fields.|&#9888; Can be cached (string with time unit, e.g. `"1h"`)|-|                                                                                               |

### OAuth2 Block

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
|[Backend Block](#backend-block)|no label|[Backend Block](#backend-block)|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `backend`                       |string|-|[Backend Block Reference](#backend-block)|-|-|ul>                                                                                                                                                                                                                                                                                 |
| `grant_type`                    |string|-|-|&#9888; required, available value(s): `client_credentials`|-|                                                                                                                                                                                                                                                                           |
| `token_endpoint`   |string|-|URL of the token endpoint at the authorization server.|&#9888; required|-|
| `client_id`|  string|-|The client identifier.|&#9888; required|-|                                                                                                                                ||
| `client_secret` |string|-|The client password.|&#9888; required.|-|
| `retries` |integer|`1` | The number of retries to get the token and resource, if the resource-request responds with `401 Unauthorized` HTTP status code.|-|-|                                                                                                                                                                      |
| `token_endpoint_auth_method` |string|`client_secret_post`|Defines the method to authenticate the client at the token endpoint.|If set to , the client credentials are transported in the request body. If set to `client_secret_basic`, the client credentials are transported via Basic Authentication.|-|
| `scope`                      |string|-|  A space separated list of requested scopes for the access token.|-|

### Definitions Block

Use the [Definitions Block](#definitions-block) to define configurations you want to reuse.
[Access Control](#access-control) is **always** defined in the [Definitions Block](#definitions-block).

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
|-|no label|[Backend Block(s)](#backend-block), [Basic Auth Block(s)](#basic-auth-block), [JWT Block(s)](#jwt-block), [JWT Signing Profile Block(s)](#jwt-signing-profile-block), [SAML Block(s)](#saml-block)|

<!--Example???-->

### Basic Auth Block

The  [Basic Auth Block](#basic-auth-block) lets you configure basic auth for your gateway. Like all
[Access Control](#access-control) types, the  [Basic Auth Block](#basic-auth-block) is defined in the
[Definitions Block](#definitions-block) and can be referenced in all configuration
blocks by its required _label_.

&#9888; If both `user`/`password` and `htpasswd_file` are configured, the incoming
credentials from the `Authorization` request HTTP header field are checked against
`user`/`password` if the user matches, and against the data in the file referenced
by `htpasswd_file` otherwise.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Definitions Block](#definitions-block)| &#9888; required |-|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `user`          |string|-|The user name.|-|-|
| `password`      |string|-|The corresponding password.|-|-|
| `htpasswd_file`      |string|-|>The htpasswd file.|-|-|
|`realm`     |string|-|The realm to be sent in a `WWW-Authenticate` response HTTP header field.|-|-|

### JWT Block

The[JWT Block](#jwt-block) lets you configure JSON Web Token access control for your gateway.
Like all [Access Control](#access-control) types, the [JWT Block](#jwt-block) is defined in
the [Definitions Block](#definitions-block) and can be referenced in all configuration blocks by its
required _label_.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Definitions Block](#definitions-block)| &#9888; required |-|
<!-- warum claims fett?-->
| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `cookie`  |string|-|Read `AccessToken` key to gain the token value from a cookie.|&#9888; available value: `AccessToken`|`cookie = "AccessToken"`|
| `header`          |string|-|-|&#9888; Implies `Bearer` if `Authorization` (case-insensitive) is used, otherwise any other header name can be used.|`header = "Authorization"` |
| `key`           |string|-|Public key (in PEM format) for `RS*` variants or the secret for `HS*` algorithm.|-|-|
| `key_file`          |string|-|Optional file reference instead of `key` usage.|-|-|
|  `signature_algorithm`           |string|-|-|&#9888; required. Valid values are: `RS256` `RS384` `RS512` `HS256` `HS384` `HS512`.|-|
| `claims`               |string|-|Equals/in comparison with JWT payload.|-|-|
| `required_claims`      | string|-|list of claims that must be given for a valid token |-|-|

### JWT Signing Profile Block

The [JWT Signing Profile Block(s)](#jwt-signing-profile-block) lets you configure a JSON Web Token signing
profile for your gateway. It is referenced in the [`jwt_sign()` function](#functions)
by its required _label_.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Definitions Block](#definitions-block)| &#9888; required |-|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
| `key`  |string|-|Private key (in PEM format) for `RS*` variants or the secret for `HS*` algorithm.|-|-|
| `key_file`  |string|-|Optional file reference instead of `key` usage.|-|-|
| `signature_algorithm`|-|-|-|&#9888; required. Valid values are: `RS256` `RS384` `RS512` `HS256` `HS384` `HS512`.|-|
|`ttl`  |string|-|The token's time-to-live (creates the `exp` claim).|-|-|
| `claims` |string|-|Default claims for the JWT payload.|-|-|

### SAML Block

The  [SAML Block(s)](#saml-block) lets you configure the `saml_sso_url()` [function](#functions) and an access
control for a SAML Assertion Consumer Service (ACS) endpoint.
Like all [Access Control](#access-control) types, the `saml` block is defined in
the [Definitions Block](#definitions-block) and can be referenced in all configuration blocks by its
required _label_.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| [Definitions Block](#definitions-block)| &#9888; required |-|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
|`idp_metadata_file`|string|-|File reference to the Identity Provider metadata XML file.|&#9888; required|-|
|`sp_acs_url`  |string|-|The URL of the Service Provider's ACS endpoint.|&#9888; required|-|
| `sp_entity_id`   |string|-|The Service Provider's entity ID.|&#9888; required|-|
| `array_attributes`|string|-|A list of assertion attributes that may have several values.|-|-|

Some information from the assertion consumed at the ACS endpoint is provided in the context at `request.context.<label>`:

- the `NameID` of the assertion's `Subject` (`request.context.<label>.sub`)
- the session expiry date `SessionNotOnOrAfter` (as UNIX timestamp: `request.context.<label>.exp`)
- the attributes (`request.context.<label>.attributes.<name>`)

### Settings Block

The `settings` block let you configure the more basic and global behavior of your
gateway instance.

|Context|Label|Nested block(s)|
| :-----------| :-----------| :-----------|
| -| no label |-|

| Attribute(s) | Type |Default|Description|Characteristic(s)| Example|
| :------------------------------ | :--------------- | :--------------- | :--------------- | :--------------- | :--------------- |
|`health_path` |string|`/healthz`| Health path which is available for all configured server and ports.|-|-|
|`no_proxy_from_env`|bool|`false`|Disables the connect hop to configured [proxy via environment](https://godoc.org/golang.org/x/net/http/httpproxy). |-|-|
|`default_port`|number|`8080`| Port which will be used if not explicitly specified per host within the [`hosts`](#server-block) list   |-|-|
|`log_format` |string|`common`|Switch for tab/field based colored view or json log lines    |-|-|
|`log_pretty`|bool|`false`|Global option for `json` log format which pretty prints with basic key coloring  |-|-|
|`xfh`  |bool|`false`|Option to use the `X-Forwarded-Host` header as the request host  |-|-|
| `request_id_format`|string|`common`| If set to `uuid4` a rfc4122 uuid is used for `request.id` and related log fields  |-|-|
|`secure_cookies`|string|`""` | If set to `"strip"`, the `Secure` flag is removed from all `Set-Cookie` HTTP header fields.    |-|-|

## Access Control

The configuration of access control is twofold in Couper: You define the particular
type (such as `jwt` or `basic_auth`) in `definitions`, each with a distinct label.
Anywhere in the `server` block those labels can be used in the `access_control`
list to protect that block. &#9888; access rights are inherited by nested blocks.
You can also disable `access_control` for blocks. By typing `disable_access_control = ["bar"]`,
the `access_control` type `bar` will be disabled for the corresponding block context.

All access controls have an option to handle related errors. Please refer to [Errors](ERRORS.md).

## Health-Check

The health check will answer a status `200 OK` on every port with the configured
`health_path`. As soon as the gateway instance will receive a `SIGINT` or `SIGTERM`
the check will return a status `500 StatusInternalServerError`. A shutdown delay
of `5s` for example allows the server to finish all running requests and gives a load-balancer
time to pick another gateway instance. After this delay the server goes into
shutdown mode with a deadline of `5s` and no new requests will be accepted.
The shutdown timings defaults to `0` which means no delaying with development setups.
Both durations can be configured via environment variable. Please refer to the [docker document](./../DOCKER.md).

## Variables

### `env`

Environment variables can be accessed everywhere within the configuration file
since these references get evaluated at start.

### `request`

| Variable                         | Description                                                                                                                                                                                                                                                                         |
| :------------------------------- | :---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `id`                             | Unique request id                                                                                                                                                                                                                                                                   |
| `method`                         | HTTP method                                                                                                                                                                                                                                                                         |
| `path`                           | URL path                                                                                                                                                                                                                                                                            |
| `endpoint`                       | Matched endpoint pattern                                                                                                                                                                                                                                                            |
| `headers.<name>`                 | HTTP request header value for requested lower-case key                                                                                                                                                                                                                              |
| `cookies.<name>`                 | Value from `Cookie` request header for requested key (&#9888; last wins!)                                                                                                                                                                                                           |
| `query.<name>`                   | Query parameter values (&#9888; last wins!)                                                                                                                                                                                                                                         |
| `path_params.<name>`             | Value from a named path parameter defined within an endpoint path label                                                                                                                                                                                                             |
| `body`                           | The request message body                                                                                                                                                                                                                                                            |
| `form_body.<name>`               | Parameter in a `application/x-www-form-urlencoded` body                                                                                                                                                                                                                             |
| `json_body.<name>`               | Access json decoded object properties. Media type must be `application/json` or `application/*+json`.                                                                                                                                                                               |
| `context.<name>.<property_name>` | Request context containing claims from JWT used for [Access Control](#access-control) or information from a SAML assertion, `<name>` being the [JWT Block's](#jwt-block) or [SAML Block's](#saml-block) label and `property_name` being the claim's or assertion information's name |

### `backend_requests`

`backend_requests.<label>` is a list of all backend requests, and their variables.
To access a specific request use the related label. [Request](#request-block) and
[Proxy](#proxy-block) blocks without a label will be available as `default`.
To access the HTTP method of the `default` request use `backend_requests.default.method` .

| Variable                         | Description                                                                                                                                                                                                                                                                         |
| :------------------------------- | :---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `id`                             | Unique request id                                                                                                                                                                                                                                                                   |
| `method`                         | HTTP method                                                                                                                                                                                                                                                                         |
| `path`                           | URL path                                                                                                                                                                                                                                                                            |
| `headers.<name>`                 | HTTP request header value for requested lower-case key                                                                                                                                                                                                                              |
| `cookies.<name>`                 | Value from `Cookie` request header for requested key (&#9888; last wins!)                                                                                                                                                                                                           |
| `query.<name>`                   | Query parameter values (&#9888; last wins!)                                                                                                                                                                                                                                         |
| `form_body.<name>`               | Parameter in a `application/x-www-form-urlencoded` body                                                                                                                                                                                                                             |
| `context.<name>.<property_name>` | Request context containing claims from JWT used for [Access Control](#access-control) or information from a SAML assertion, `<name>` being the [JWT Block's](#jwt-block) or [SAML Block's](#saml-block) label and `property_name` being the claim's or assertion information's name |
| `url`                            | Backend origin URL                                                                                                                                                                                                                                                                  |

### `backend_responses`

`backend_responses.<label>` is a list of all backend responses, and their variables. Same behaviour as for `backend_requests`.
Use the related label to access a specific response.
[Request](#request-block) and [Proxy](#proxy-block) blocks without a label will be available as `default`.
To access the HTTP status code of the `default` response use `backend_responses.default.status` .

| Variable           | Description                                                                                           |
| :----------------- | :---------------------------------------------------------------------------------------------------- |
| `status`           | HTTP status code                                                                                      |
| `headers.<name>`   | HTTP response header value for requested lower-case key                                               |
| `cookies.<name>`   | Value from `Set-Cookie` response header for requested key (&#9888; last wins!)                        |
| `body`             | The response message body                                                                             |
| `json_body.<name>` | Access json decoded object properties. Media type must be `application/json` or `application/*+json`. |

## Functions

| Name            | Description                                                                                                                                                                                                                                                                                          |
| :-------------- | :--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `base64_decode` | Decodes Base64 data, as specified in RFC 4648.                                                                                                                                                                                                                                                       |
| `base64_encode` | Encodes Base64 data, as specified in RFC 4648.                                                                                                                                                                                                                                                       |
| `coalesce`      | Returns the first of the given arguments that is not null.                                                                                                                                                                                                                                           |
| `json_decode`   | Parses the given JSON string and, if it is valid, returns the value it represents.                                                                                                                                                                                                                   |
| `json_encode`   | Returns a JSON serialization of the given value.                                                                                                                                                                                                                                                     |
| `jwt_sign`      | jwt_sign creates and signs a JSON Web Token (JWT) from information from a referenced [JWT Signing Profile Block](#jwt-signing-profile-block) and additional claims provided as a function parameter.                                                                                                 |
| `merge`         | Deep-merges two or more of either objects or tuples. `null` arguments are ignored. A `null` attribute value in an object removes the previous attribute value. An attribute value with a different type than the current value is set as the new value. `merge()` with no parameters returns `null`. |
| `saml_sso_url`  | Creates a SAML SingleSignOn URL (including the `SAMLRequest` parameter) from a referenced [SAML Block](#saml-block).                                                                                                                                                                                 |
| `to_lower`      | Converts a given string to lowercase.                                                                                                                                                                                                                                                                |
| `to_upper`      | Converts a given string to uppercase.                                                                                                                                                                                                                                                                |
| `unixtime`      | Retrieves the current UNIX timestamp in seconds.                                                                                                                                                                                                                                                     |
| `url_encode`    | URL-encodes a given string according to RFC 3986.                                                                                                                                                                                                                                                    |

## Modifier

- [Request Header](#request-header)
- [Response Header](#response-header)
- [Set Response Status](#set-response-status)
- [Query Parameter](#query-parameter)
- [Form Parameter](#form-parameter)

### Request Header

Couper offers three attributes to manipulate the request header fields. The header
attributes can be defined unordered within the configuration file but will be
executed ordered as follows:

| Modifier                 | Contexts                                                                                                                                                | Description                                                       |
| :----------------------- | :------------------------------------------------------------------------------------------------------------------------------------------------------ | :---------------------------------------------------------------- |
| `remove_request_headers` | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | list of request header to be removed from the upstream request.   |
| `set_request_headers`    | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | Key/value(s) pairs to set request header in the upstream request. |
| `add_request_headers`    | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | Key/value(s) pairs to add request header to the upstream request. |

All `*_request_headers` are executed from: `endpoint`, `proxy`, `backend` and `error_handler`.

### Response Header

Couper offers three attributes to manipulate the response header fields. The header
attributes can be defined unordered within the configuration file but will be
executed ordered as follows:

| Modifier                  | Contexts                                                                                                                                                                                                                                                              | Description                                                       |
| :------------------------ | :-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- | :---------------------------------------------------------------- |
| `remove_response_headers` | [Server Block](#server-block), [Files Block](#files-block), [SPA Block](#spa-block), [API Block](#api-block), [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | list of response header to be removed from the client response.   |
| `set_response_headers`    | [Server Block](#server-block), [Files Block](#files-block), [SPA Block](#spa-block), [API Block](#api-block), [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | Key/value(s) pairs to set response header in the client response. |
| `add_response_headers`    | [Server Block](#server-block), [Files Block](#files-block), [SPA Block](#spa-block), [API Block](#api-block), [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | Key/value(s) pairs to add response header to the client response. |

All `*_response_headers` are executed from: `server`, `files`, `spa`, `api`, `endpoint`, `proxy`, `backend` and `error_handler`.

### Set Response Status

The `set_response_status` attribute allows to modify the HTTP status code to the
given value.

| Modifier              | Contexts                                                                                            | Description                                        |
| :-------------------- | :-------------------------------------------------------------------------------------------------- | :------------------------------------------------- |
| `set_response_status` | [Endpoint Block](#endpoint-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | HTTP status code to be set to the client response. |

If the HTTP status code ist set to `204`, the reponse body and the HTTP header
field `Content-Length` is removed from the client response, and a warning is logged.

## Parameters

### Query Parameter

Couper offers three attributes to manipulate the query parameter. The query
attributes can be defined unordered within the configuration file but will be
executed ordered as follows:

| Modifier              | Contexts                                                                                                                                                | Description                                                             |
| :-------------------- | :------------------------------------------------------------------------------------------------------------------------------------------------------ | :---------------------------------------------------------------------- |
| `remove_query_params` | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | list of query parameters to be removed from the upstream request URL.   |
| `set_query_params`    | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | Key/value(s) pairs to set query parameters in the upstream request URL. |
| `add_query_params`    | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | Key/value(s) pairs to add query parameters to the upstream request URL. |

All `*_query_params` are executed from: `endpoint`, `proxy`, `backend` and `error_handler`.

```hcl
server "my_project" {
  api {
    endpoint "/" {
      proxy {
        backend = "example"
      }
    }
  }
}

definitions {
  backend "example" {
    origin = "http://example.com"

    remove_query_params = ["a", "b"]

    set_query_params = {
      string = "string"
      multi = ["foo", "bar"]
      "${request.headers.example}" = "yes"
    }

    add_query_params = {
      noop = request.headers.noop
      null = null
      empty = ""
    }
  }
}
```

### Form Parameter

Couper offers three attributes to manipulate the form parameter. The form
attributes can be defined unordered within the configuration file but will be
executed ordered as follows:

| Modifier             | Contexts                                                                                                                                                | Description                                                             |
| :------------------- | :------------------------------------------------------------------------------------------------------------------------------------------------------ | :---------------------------------------------------------------------- |
| `remove_form_params` | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | list of form parameters to be removed from the upstream request body.   |
| `set_form_params`    | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | Key/value(s) pairs to set form parameters in the upstream request body. |
| `add_form_params`    | [Endpoint Block](#endpoint-block), [Proxy Block](#proxy-block), [Backend Block](#backend-block), [Error Handler](ERRORS.md#error_handler-specification) | Key/value(s) pairs to add form parameters to the upstream request body. |

All `*_form_params` are executed from: `endpoint`, `proxy`, `backend` and `error_handler`.

The `*_form_params` apply only to requests with the `POST` method and
the `Content-Type: application/x-www-form-urlencoded` HTTP header field.

```hcl
server "my_project" {
  api {
    endpoint "/" {
      proxy {
        backend = "example"
      }
    }
  }
}

definitions {
  backend "example" {
    origin = "http://example.com"

    remove_form_params = ["a", "b"]

    set_form_params = {
      string = "string"
      multi = ["foo", "bar"]
      "${request.headers.example}" = "yes"
    }

    add_form_params = {
      noop = request.headers.noop
      null = null
      empty = ""
    }
  }
}
```

### Path Parameter

An endpoint label could be defined as `endpoint "/app/{section}/{project}/view" { ... }`
to access the named path parameter `section` and `project` via `request.path_params.*`.
The values would map as following for the request path: `/app/nature/plant-a-tree/view`:

| Variable                      | Value          |
| :---------------------------- | :------------- |
| `request.path_params.section` | `nature`       |
| `request.path_params.project` | `plant-a-tree` |
